//
//  tableViewController.swift
//  CoreDataDemo
//
//  Created by Sarah Abraham on 9/7/16.
//  Copyright © 2016 Sarah Abraham. All rights reserved.
//

import UIKit
import CoreData

class TableViewController: UITableViewController {

    @IBOutlet weak var addPersonButton: UIBarButtonItem!
    
    var names = [String]()
    var occupations = [String]()
    
    var people = [NSManagedObject]()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        names += ["Frodo", "Gandalf", "Aragorn", "Galadriel", "Arwen", "Eowyn", "Sam", "Boromir"]
        occupations += ["Ring-bearer", "Wizard", "Ranger", "Ruler", "Warrior", "Gardener"]
        
        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false
        
        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        let managedContext = appDelegate.managedObjectContext
        let fetchRequest = NSFetchRequest<NSFetchRequestResult>(entityName:"Person")
        var fetchedResults:[NSManagedObject]? = nil
        
        do {
            try fetchedResults = managedContext.fetch(fetchRequest) as? [NSManagedObject]
        } catch {
            let nserror = error as NSError
            NSLog("Unable to fetch \(nserror), \(nserror.userInfo)")
            abort()
        }
        
        if let results = fetchedResults {
            people = results
        }

    }
    
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    @IBAction func addAPerson(sender: UIBarButtonItem) {
        let name = names[Int(arc4random_uniform(UInt32(names.count)))]
        let age = Int(arc4random_uniform(UInt32(500)))
        let occupation = occupations[Int(arc4random_uniform(UInt32(occupations.count)))]

        addPerson(name: name, occupation: occupation, age: age)
        self.tableView.reloadData()
    }
    
    // MARK: - Table view data source
    
    override func numberOfSections(in tableView: UITableView) -> Int {
        // Return the number of sections
        return 1
    }
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Return the number of rows
        return people.count
    }
    
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cellIdentifier = "TableViewCell"
        let cell = tableView.dequeueReusableCell(withIdentifier: cellIdentifier, for: indexPath) as! TableViewCell
        
        let person = people[indexPath.row]
        
        cell.cellName.text = person.value(forKey: "name") as? String
        let age:Int = person.value(forKey: "age") as! Int
        cell.cellAge.text = String(age)
        cell.cellOccupation.text = person.value(forKey: "occupation") as? String
        
        return cell
    }
    
    func addPerson(name: String, occupation: String, age: Int) {
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        
        let managedContext = appDelegate.managedObjectContext
        
        let entity = NSEntityDescription.entity(forEntityName: "Person", in: managedContext)
        
        let person = NSManagedObject(entity: entity!, insertInto: managedContext)
        
        person.setValue(name, forKey: "name")
        person.setValue(age, forKey: "age")
        person.setValue(occupation, forKey: "occupation")
        
        do {
            try managedContext.save()
        } catch {
            let nserror = error as NSError
            NSLog("Unable to save \(nserror), \(nserror.userInfo)")
            abort()
        }
        
        people.append(person)
    }
    
}
